using System;
using System.Collections.Generic;
using System.Linq;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Media;

namespace ChristmasSpecial
{

    public class ChristmasSpecial : Microsoft.Xna.Framework.Game
    {
        GraphicsDeviceManager graphics;
        SpriteBatch spriteBatch;

        VisualizationData visData = new VisualizationData();

        Texture2D gfxBackground;
        Texture2D gfxBackground2;
        Texture2D gfxForeground;
        SpriteFont gfxFont;
        Song song;
        int screenWidth;
        int screenHeight;

        float parallax1Offset = 0;
        float parallax2Offset = 0;
        float textOffset = 0;
        float angle = 0;
        Vector2 textPosition;

        string text = "MERRY CHRISTMAS 2014! AND A HAPPY NEW YEAR!";
        string[] texts;
        bool boom;
        float bass;
        Color colorFilter;

        public ChristmasSpecial()
        {
            graphics = new GraphicsDeviceManager(this);
            Content.RootDirectory = "Content";
        }

        protected override void Initialize()
        {
            screenHeight = graphics.GraphicsDevice.Viewport.Height;
            screenWidth = graphics.GraphicsDevice.Viewport.Width;

            texts = text.Split(new char[] {' '});
            colorFilter = Color.White;

            base.Initialize();
        }

        protected override void LoadContent()
        {
            spriteBatch = new SpriteBatch(GraphicsDevice);

            gfxBackground = Content.Load<Texture2D>("background");
            gfxBackground2 = Content.Load<Texture2D>("background2");
            gfxForeground = Content.Load<Texture2D>("foreground");
            gfxFont = Content.Load<SpriteFont>("xmasfont");
            song = Content.Load<Song>("xmas");

            MediaPlayer.IsVisualizationEnabled = true;
            MediaPlayer.IsRepeating = true;
            MediaPlayer.Play(song);
        }

        protected override void UnloadContent()
        {
            // TODO: Unload any non ContentManager content here
        }


        protected override void Update(GameTime gameTime)
        {
            if (GamePad.GetState(PlayerIndex.One).Buttons.Back == ButtonState.Pressed)
                this.Exit();

            if (MediaPlayer.State == MediaState.Playing)
            {
                bass = 0;
                MediaPlayer.GetVisualizationData(visData);
                for(int i=40; i<56; i++)
                    bass += visData.Frequencies[i];
            }

            if (bass > 8.4f)
                colorFilter = new Color(128, 128, 128);
            else if (colorFilter.B < 254)
                colorFilter = new Color(colorFilter.B + 2, colorFilter.B + 2, colorFilter.B + 2);


            parallax1Offset += 0.20f;
            if (parallax1Offset > 190)
                parallax1Offset = parallax1Offset - 190;

            parallax2Offset += 2.00f;
            if (parallax2Offset > 1350)
                parallax2Offset = parallax2Offset - 1350;

            textOffset += 2.0f;
            angle += 0.1f;

            textPosition = new Vector2(800 - textOffset, 250);
            if (textOffset > 4000)
                textOffset = textOffset - 4000;

            base.Update(gameTime);
        }

        protected override void Draw(GameTime gameTime)
        {
            GraphicsDevice.Clear(Color.CornflowerBlue);

            spriteBatch.Begin(SpriteSortMode.Deferred, BlendState.AlphaBlend, SamplerState.PointClamp, DepthStencilState.Default, RasterizerState.CullNone);
            
            Rectangle screenRect =  new Rectangle(0, 0, screenWidth, screenHeight);
            Rectangle parallax1 =  new Rectangle((int)parallax1Offset, 0, screenWidth / 2, gfxBackground2.Height);
            Rectangle parallax2 = new Rectangle((int)parallax2Offset, 0, screenWidth / 2, gfxForeground.Height);

            spriteBatch.Draw(gfxBackground, screenRect, colorFilter);
            spriteBatch.Draw(gfxBackground2, new Vector2(0, 240), parallax1, Color.White, 0f, Vector2.Zero, 2.0f, SpriteEffects.None, 0f);

            Vector2 space = Vector2.Zero;
            var a = angle;
            foreach (char c in text)
            {
                var width = gfxFont.MeasureString(c.ToString());
                var offset = new Vector2(20 - width.X, 0) * 1.5f;
                space += new Vector2(20, 0) * 3;
                float rotation = (float)Math.Cos(a) / 4;
                spriteBatch.DrawString(gfxFont, c.ToString(), textPosition + new Vector2(space.X, 60 * (float)Math.Sin(a)) + offset, Color.White, rotation, width / 2, 3.0f, SpriteEffects.None, 0f);
                a += 0.3f;
            }


            //int y = 0;
            //foreach(var f in visData.Frequencies)
            //{
            //    spriteBatch.Draw(gfxBackground2, new Rectangle(0, y, (int)(f * 200), 1), new Rectangle(10, 50, 1, 1), Color.White);
            //    y++;
            //}

            //spriteBatch.DrawString(gfxFont, bass.ToString(), new Vector2(10, 20), Color.White);
            spriteBatch.Draw(gfxForeground, new Vector2(0, 200), parallax2, Color.White, 0f, Vector2.Zero, 2.0f, SpriteEffects.None, 0f);
            spriteBatch.End();

            base.Draw(gameTime);
        }
    }
}
